/*
 * Copyright (c) 2000 Paul Herman
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "tcpprof.h"

char str[1024];
struct ether_addr _u_ether_addr;

#ifndef xtoa
#define xtoa(x)         (u_long)(((x)<'a')? ((x)-'0') : ((x)-'W'))
#endif

long my_strtol16(char *c) {
	if (strlen(c) == 2)
		return (xtoa(tolower(c[0]))<<4) + xtoa(tolower(c[1]));
	else return xtoa(tolower(c[0]));
}

/*
 * wrote this to replace strsep(3) because I noticed from gprof that
 * this is called *very* often...  needs to be fast.
 */
char *my_strsep(char **p, char d) {
	register char *r, *s;

	s = *p;

	while (*s != d && *s && *s != '\n') s++;
	r = (*s)? *p : NULL;
	*s++ = 0;
	*p = s;
	return r;
}

char *getfirstchar(char *s) {
	while (*s == ' ' || *s == '\t') s++;
	return s;
}

int is_ether_addr(char *c) {
	char *p, cc = 0;
	p = c;
	while (*p) {
		if (*p == ':') cc++;
		if (*p != ':' &&
			(*p < '0' || *p > '9') &&
			(*p < 'a' || *p > 'z') &&
			(*p < 'A' || *p > 'Z') )
			return 0;
		else p++;
		}
	if (cc == ETHER_ADDR_LEN-1) return 1;
	else return 0;
}

#ifndef xtoa
#define xtoa(x)		(u_long)(((x)<'a')? ((x)-'0') : ((x)-'W'))
#endif

/* NEEDS WORK! */
struct ether_addr *my_ether_aton(char *c) {
	char tmp[128], *a, *b;
	u_char *e;

	if (!is_ether_addr(c)) return NULL;

	strcpy(tmp, c);
	a = tmp;
	e = (u_char *)&_u_ether_addr;
	while ((b = my_strsep(&a, ':')) != NULL) {
/*		*e++ = strtol(b, NULL, 16); */
		*e++ = my_strtol16(b);
		}
	return &_u_ether_addr;
}

char *my_basename(char *p) {
	char *s;

	s = p;
	if (!*s || strlen(s) < 2) return p;

	for (s += strlen(p); s>p && *s != '/'; s--) continue;
	return (*s == '/')? s+1 : s;
}

void timeval_diff(struct timeval *d, struct timeval *t2, struct timeval *t1) {
	d->tv_sec  = t2->tv_sec  - t1->tv_sec;
	d->tv_usec = t2->tv_usec - t1->tv_usec;
}

void timeval_avg(struct timeval *d, struct timeval *t2, struct timeval *t1) {
	d->tv_sec  = (time_t)((quad_t)t2->tv_sec  + (quad_t)t1->tv_sec)/2;
	d->tv_usec = (time_t)((quad_t)t2->tv_usec + (quad_t)t1->tv_usec)/2;
}

int timeval_lessthan(struct timeval *t1, struct timeval *t2) {
	if (t1->tv_sec < t2->tv_sec) return 1;
	if (t1->tv_sec > t2->tv_sec) return 0;
	if (t1->tv_usec < t2->tv_usec) return 1;
	return 0;
}

int timeval_lessthaneq(struct timeval *t1, struct timeval *t2) {
	if (t1->tv_sec < t2->tv_sec) return 1;
	if (t1->tv_sec > t2->tv_sec) return 0;
	if (t1->tv_usec > t2->tv_usec) return 0;
	return 1;
}

void strtoargv(char **argv, char *s, int *idx) {
	char *p;
	while ( (p = strsep(&s, " ")) != NULL) {
		argv[(*idx)++] = p;
		}
}

u_char count_1bits(u_int x) {
	u_char r = 0;
	int i;

	for (i=0; i<8*sizeof(x); i++)
		if ( (x>>i) & 0x1 ) r++;
	return r;
};

char *my_get_port(u_short port, int n_only) {
	struct servent *srvent;
	if (n_only) {
		sprintf(str, "%u", port);
		}
	else {
	        srvent = getservbyport(htons(port), NULL);
		if (srvent == NULL) sprintf(str, "%u", port);
		else	sprintf(str, "%s (%u)", srvent->s_name, port);
		}
	return str;
}

char *my_get_proto(u_short proto, int n_only) {
	struct protoent *ip_prot;

	if (n_only) {
		sprintf(str, "%u", proto);
		}
	else {
	        ip_prot = getprotobynumber(proto);
		if (ip_prot == NULL) sprintf(str, "%u", proto);
		else	sprintf(str, "%s", ip_prot->p_name);
		}
	return str;
}

int key_compare(const void *h1, const void *h2) {
	const stat_unit **aa, **bb;

	aa = (const stat_unit **)h1; bb = (const stat_unit **)h2;
	return ((*aa)->key < (*bb)->key)? -1 : 
		((*aa)->key > (*bb)->key)? 1 : 0;
}

int bytes_compare(const void *h1, const void *h2) {
	const stat_unit **aa, **bb;

	aa = (const stat_unit **)h1; bb = (const stat_unit **)h2;

		/* It's reversed here ... */
	return ((*aa)->bytes < (*bb)->bytes)? 1 : 
		((*aa)->bytes > (*bb)->bytes)? -1 : 0;
}

char ether_addr_is_zero(u_char addr[ETHER_ADDR_LEN]) {
	char ret = 1, i;
	for(i=0; i<ETHER_ADDR_LEN; i++) ret &= (addr[i]!=0)? 1 : 0;
	return ret;
}
