/*
 * Copyright (c) 2000 Paul Herman
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "tcpprof.h"

extern char filterexpr[];
packet_data	my_packet_data;

struct	hook_and_sinker {
	void 	(*hook)(packet_data *, void **);
	void	**args;
	int	proc_flags;
	bpf_u_int32	linktype;
} hook_and_sinker;

void process_pcap(u_char *user, const struct pcap_pkthdr *h, const u_char *p) {
	struct hook_and_sinker	*hs;
	struct ether_header	*ep;
	u_int length = h->caplen;
	u_char	*packet;

	hs = (struct hook_and_sinker *) user;
	packet = (u_char *) p;
	ep = (struct ether_header *) p;

	my_packet_data.link_type = NONE;
	switch (hs->linktype) {
		case DLT_NULL:
		packet += sizeof(int);	/* dunno what this is... */
		break;

		case DLT_EN10MB:
		packet += sizeof(struct ether_header);
		length -= sizeof(struct ether_header);
		bcopy(&(ep->ether_shost), &(my_packet_data.ether.ether_shost), sizeof(struct ether_addr));
		bcopy(&(ep->ether_dhost), &(my_packet_data.ether.ether_dhost), sizeof(struct ether_addr));
		switch (ntohs(ep->ether_type)) {
	        	case ETHERTYPE_IP:
				my_packet_data.link_type = ETHERNET_IP;
			 	break;
	        	case ETHERTYPE_ARP:
				my_packet_data.link_type = ETHERNET_ARP;
				break;
			default:
        			break;
			}
		my_packet_data.ether.ether_type = ep->ether_type;
		break;
		}

	bcopy((void *)&(h->ts), &(my_packet_data.timestamp), sizeof(struct timeval) );
	bcopy(packet, &(my_packet_data.data.raw), (length<PAK_SIZ)? length : PAK_SIZ);

	if (my_packet_data.link_type == ETHERNET_IP &&
		!(hs->proc_flags & GET_TCPD_COUNT_LINKSIZE) )
		my_packet_data.packet_len = h->len - ETHER_HDR_LEN;
	else	my_packet_data.packet_len = h->len;

	hs->hook(&my_packet_data, hs->args);
}

/*
 * get_tcpdump_data() is a hook for pcap_loop and
 *   calls a user function pointing to the data
 */
void get_tcpdump_data(char *fname, char *filter, int flags,
	Double capture_seconds, void (*hook)(packet_data *, void **),
	void **args) {

	pcap_t	*pd;
	char	ebuf[PCAP_ERRBUF_SIZE];
	struct	bpf_program bpf_prog;
	struct hook_and_sinker	hs;

#define SNAPLEN	68	/* XXX: Doesn't belong here */
	if (flags & GET_TCPD_DO_LIVE) {
		if (fname == NULL || 
		 (strlen(fname) == strlen("auto") &&
			strncmp(fname, "auto", 4) == 0)
			) {
			fname = pcap_lookupdev(ebuf);
			if (fname == NULL) err(1, ebuf);
			}
		pd = pcap_open_live(fname, SNAPLEN,
			flags & GET_TCPD_DO_LIVE_PROMISC, 1000, ebuf);
		}
		
	else pd = pcap_open_offline(fname, ebuf);
	if (pd == NULL) err(1, ebuf);
	if (pcap_compile(pd, &bpf_prog, filter, 1, 0) < 0) err(1, pcap_geterr(pd));
	if (pcap_setfilter(pd, &bpf_prog) < 0) err(1, pcap_geterr(pd));

		/* XXX: Check the Datalink!!!! if not supported, then fail */	
			/* pcap_datalink() see tcpdump.c */
	hs.hook = hook;
	hs.args = args;
	hs.proc_flags = flags;
	hs.linktype = pcap_datalink(pd);

	if (pcap_loop(pd, -1, process_pcap, (u_char *)&hs) < 0) err(1, pcap_geterr(pd));
}
