/*
 * Copyright (c) 2000 Paul Herman
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "tcpprof.h"

char str[1024];
struct ether_addr _u_ether_addr;

char *my_basename(char *p) {
	char *s;

	s = p;
	if (!*s || strlen(s) < 2) return p;

	for (s += strlen(p); s>p && *s != '/'; s--) continue;
	return (*s == '/')? s+1 : s;
}

void timeval_diff(struct timeval *d, struct timeval *t2, struct timeval *t1) {
	d->tv_sec  = t2->tv_sec  - t1->tv_sec;
	d->tv_usec = t2->tv_usec - t1->tv_usec;
}

void timeval_avg(struct timeval *d, struct timeval *t2, struct timeval *t1) {
	d->tv_sec  = (time_t)((quad_t)t2->tv_sec  + (quad_t)t1->tv_sec)/2;
	d->tv_usec = (time_t)((quad_t)t2->tv_usec + (quad_t)t1->tv_usec)/2;
}

int timeval_lessthan(struct timeval *t1, struct timeval *t2) {
	if (t1->tv_sec < t2->tv_sec) return 1;
	if (t1->tv_sec > t2->tv_sec) return 0;
	if (t1->tv_usec < t2->tv_usec) return 1;
	return 0;
}

int timeval_lessthaneq(struct timeval *t1, struct timeval *t2) {
	if (t1->tv_sec < t2->tv_sec) return 1;
	if (t1->tv_sec > t2->tv_sec) return 0;
	if (t1->tv_usec > t2->tv_usec) return 0;
	return 1;
}

void strtoargv(char **argv, char *s, int *idx) {
	char *p;
	while ( (p = strsep(&s, " ")) != NULL) {
		argv[(*idx)++] = p;
		}
}

u_char count_1bits(u_int x) {
	u_char r = 0;
	int i;

	for (i=0; i<8*sizeof(x); i++)
		if ( (x>>i) & 0x1 ) r++;
	return r;
};

char *my_get_port(u_short port, int n_only) {
	struct servent *srvent;
	if (n_only) {
		sprintf(str, "%u", port);
		}
	else {
	        srvent = getservbyport(htons(port), NULL);
		if (srvent == NULL) sprintf(str, "%u", port);
		else	sprintf(str, "%s (%u)", srvent->s_name, port);
		}
	return str;
}

char *my_get_proto(u_short proto, int n_only) {
	struct protoent *ip_prot;

	if (n_only) {
		sprintf(str, "%u", proto);
		}
	else {
	        ip_prot = getprotobynumber(proto);
		if (ip_prot == NULL) sprintf(str, "%u", proto);
		else	sprintf(str, "%s", ip_prot->p_name);
		}
	return str;
}

int key_compare(const void *h1, const void *h2) {
	const stat_unit **aa, **bb;

	aa = (const stat_unit **)h1; bb = (const stat_unit **)h2;
	return ((*aa)->key < (*bb)->key)? -1 : 
		((*aa)->key > (*bb)->key)? 1 : 0;
}

int bytes_compare(const void *h1, const void *h2) {
	const stat_unit **aa, **bb;

	aa = (const stat_unit **)h1; bb = (const stat_unit **)h2;

		/* It's reversed here ... */
	return ((*aa)->bytes < (*bb)->bytes)? 1 : 
		((*aa)->bytes > (*bb)->bytes)? -1 : 0;
}

char ether_addr_is_zero(u_char addr[ETHER_ADDR_LEN]) {
	char ret = 1;
	int i;
	for(i=0; i<ETHER_ADDR_LEN; i++) ret &= (addr[i]!=0)? 1 : 0;
	return ret;
}

	/* XXX: Does this define belong here? */
#define MAX_STRCPY	32768

char *my_safe_strcpy(char **dst, const char *src) {
	size_t n;

	n = strlen(src)*sizeof(char);
	if (n<1) n = 1;
	if (n>MAX_STRCPY) n = MAX_STRCPY;

	if (*dst != NULL) free(*dst);
	*dst = (char *)malloc(n+1);
	**dst = '\0';
	return strncat(*dst, src, n);
}
